from stac_check.lint import Linter


def test_lint_dict_collection():
    file = {
        "id": "simple-collection",
        "type": "Collection",
        "stac_extensions": [
            "https://stac-extensions.github.io/eo/v1.0.0/schema.json",
            "https://stac-extensions.github.io/projection/v1.0.0/schema.json",
            "https://stac-extensions.github.io/view/v1.0.0/schema.json",
        ],
        "stac_version": "1.0.0",
        "description": "A simple collection demonstrating core catalog fields with links to a couple of items",
        "title": "Simple Example Collection",
        "providers": [
            {
                "name": "Remote Data, Inc",
                "description": "Producers of awesome spatiotemporal assets",
                "roles": ["producer", "processor"],
                "url": "http://remotedata.io",
            }
        ],
        "extent": {
            "spatial": {
                "bbox": [
                    [
                        172.91173669923782,
                        1.3438851951615003,
                        172.95469614953714,
                        1.3690476620161975,
                    ]
                ]
            },
            "temporal": {
                "interval": [["2020-12-11T22:38:32.125Z", "2020-12-14T18:02:31.437Z"]]
            },
        },
        "license": "CC-BY-4.0",
        "summaries": {
            "platform": ["cool_sat1", "cool_sat2"],
            "constellation": ["ion"],
            "instruments": ["cool_sensor_v1", "cool_sensor_v2"],
            "gsd": {"minimum": 0.512, "maximum": 0.66},
            "eo:cloud_cover": {"minimum": 1.2, "maximum": 1.2},
            "proj:epsg": {"minimum": 32659, "maximum": 32659},
            "view:sun_elevation": {"minimum": 54.9, "maximum": 54.9},
            "view:off_nadir": {"minimum": 3.8, "maximum": 3.8},
            "view:sun_azimuth": {"minimum": 135.7, "maximum": 135.7},
        },
        "links": [
            {
                "rel": "root",
                "href": "./collection.json",
                "type": "application/json",
                "title": "Simple Example Collection",
            },
            {
                "rel": "item",
                "href": "./simple-item.json",
                "type": "application/geo+json",
                "title": "Simple Item",
            },
            {"rel": "item", "href": "./core-item.json", "type": "application/geo+json"},
            {
                "rel": "item",
                "href": "./extended-item.json",
                "type": "application/geo+json",
                "title": "Extended Item",
            },
        ],
    }
    linter = Linter(file)
    assert linter.valid_stac == True
    assert linter.asset_type == "COLLECTION"
    assert linter.check_catalog_file_name() == True


def test_lint_dict_item():
    file = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "20201211_223832_CS2",
        "bbox": [
            172.91173669923782,
            1.3438851951615003,
            172.95469614953714,
            1.3690476620161975,
        ],
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [172.91173669923782, 1.3438851951615003],
                    [172.95469614953714, 1.3438851951615003],
                    [172.95469614953714, 1.3690476620161975],
                    [172.91173669923782, 1.3690476620161975],
                    [172.91173669923782, 1.3438851951615003],
                ]
            ],
        },
        "properties": {
            "title": "Core Item",
            "description": "A sample STAC Item that includes examples of all common metadata",
            "datetime": None,
            "start_datetime": "2020-12-11T22:38:32.125Z",
            "end_datetime": "2020-12-11T22:38:32.327Z",
            "created": "2020-12-12T01:48:13.725Z",
            "updated": "2020-12-12T01:48:13.725Z",
            "platform": "cool_sat1",
            "instruments": ["cool_sensor_v1"],
            "constellation": "ion",
            "mission": "collection 5624",
            "gsd": 0.512,
        },
        "collection": "simple-collection",
        "links": [
            {
                "rel": "collection",
                "href": "./collection.json",
                "type": "application/json",
                "title": "Simple Example Collection",
            },
            {
                "rel": "root",
                "href": "./collection.json",
                "type": "application/json",
                "title": "Simple Example Collection",
            },
            {
                "rel": "parent",
                "href": "./collection.json",
                "type": "application/json",
                "title": "Simple Example Collection",
            },
            {
                "rel": "alternate",
                "type": "text/html",
                "href": "http://remotedata.io/catalog/20201211_223832_CS2/index.html",
                "title": "HTML version of this STAC Item",
            },
        ],
        "assets": {
            "analytic": {
                "href": "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2_analytic.tif",
                "type": "image/tiff; application=geotiff; profile=cloud-optimized",
                "title": "4-Band Analytic",
                "roles": ["data"],
            },
            "thumbnail": {
                "href": "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.jpg",
                "title": "Thumbnail",
                "type": "image/png",
                "roles": ["thumbnail"],
            },
            "visual": {
                "href": "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.tif",
                "type": "image/tiff; application=geotiff; profile=cloud-optimized",
                "title": "3-Band Visual",
                "roles": ["visual"],
            },
            "udm": {
                "href": "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2_analytic_udm.tif",
                "title": "Unusable Data Mask",
                "type": "image/tiff; application=geotiff;",
            },
            "json-metadata": {
                "href": "http://remotedata.io/catalog/20201211_223832_CS2/extended-metadata.json",
                "title": "Extended Metadata",
                "type": "application/json",
                "roles": ["metadata"],
            },
            "ephemeris": {
                "href": "http://cool-sat.com/catalog/20201211_223832_CS2/20201211_223832_CS2.EPH",
                "title": "Satellite Ephemeris Metadata",
            },
        },
    }
    linter = Linter(file)
    assert linter.valid_stac == True
    assert linter.asset_type == "ITEM"
    assert linter.check_datetime_null() == True
    assert linter.create_best_practices_dict()["datetime_null"] == [
        "Please avoid setting the datetime field to null, many clients search on this field"
    ]



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

# -- Options for including images from README ----------------------------------
import os
import shutil

project = "stac-check"
copyright = "2025, Jonathan Healy"
author = "Jonathan Healy"
release = "1.11.1"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.napoleon",
    "sphinx_rtd_theme",
    "sphinx.ext.intersphinx",
    "myst_parser",
    "sphinx_click",
]

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
html_static_path = []

# Create _static directory if it doesn't exist
static_dir = os.path.join(os.path.dirname(__file__), "_static")
if not os.path.exists(static_dir):
    os.makedirs(static_dir)

# Copy assets from project root to _static directory
assets_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), "assets")
if os.path.exists(assets_dir):
    for file in os.listdir(assets_dir):
        src = os.path.join(assets_dir, file)
        dst = os.path.join(static_dir, file)
        if os.path.isfile(src):
            shutil.copy2(src, dst)

# Now that we've copied files, update static path
html_static_path = ["_static"]

myst_heading_anchors = 3  # Generate anchors for h1, h2, and h3
suppress_warnings = ["myst.header", "myst.xref_missing"]

# Configure myst-parser to handle images
myst_url_schemes = ("http", "https", "mailto", "ftp")

html_css_files = [
    "custom.css",
]



import pytest

from stac_check.lint import Linter

# Check if stac-pydantic is available
try:
    import importlib

    importlib.import_module("stac_pydantic")
    PYDANTIC_AVAILABLE = True
except ImportError:
    PYDANTIC_AVAILABLE = False

# Test decorator for pydantic tests
pytest.mark.pydantic = pytest.mark.skipif(
    not PYDANTIC_AVAILABLE,
    reason="stac-pydantic is not installed. Run 'pip install -e .[dev]' to install test dependencies.",
)


@pytest.mark.pydantic
def test_lint_pydantic_validation_valid():
    """Test pydantic validation with a valid STAC item."""
    file = "sample_files/1.0.0/core-item.json"
    linter = Linter(file, pydantic=True)

    assert linter.valid_stac is True
    assert linter.asset_type == "ITEM"
    assert any("stac-pydantic" in schema for schema in linter.message["schema"])
    assert linter.message["validation_method"] == "pydantic"


@pytest.mark.pydantic
def test_lint_pydantic_validation_invalid():
    """Test pydantic validation with an invalid STAC item (missing required fields)."""
    file = "sample_files/1.0.0/bad-item.json"
    linter = Linter(file, pydantic=True)

    assert linter.valid_stac is False
    assert "PydanticValidationError" in linter.message["error_type"]
    assert "id: Field required" in linter.message["error_message"]


def test_pydantic_fallback_without_import(monkeypatch):
    """Test that pydantic validation falls back to JSONSchema when stac-pydantic is not available."""
    # Skip this test if stac-pydantic is actually installed
    if PYDANTIC_AVAILABLE:
        pytest.skip("stac-pydantic is installed, skipping fallback test")

    # Test that pydantic=False works without stac-pydantic
    file = "sample_files/1.0.0/core-item.json"
    linter = Linter(file, pydantic=False)
    assert linter.valid_stac is True
    assert linter.asset_type == "ITEM"
    assert linter.message["validation_method"] == "default"

    # Test that pydantic=True falls back to JSONSchema when stac-pydantic is not available
    linter = Linter(file, pydantic=True)
    assert linter.valid_stac is True
    assert linter.asset_type == "ITEM"
    assert linter.message["validation_method"] == "default"



from stac_check.lint import Linter


def test_lint_assets_no_links():
    file = "sample_files/1.0.0/core-item.json"
    linter = Linter(file, assets=True, assets_open_urls=False)
    assert linter.message == {
        "version": "1.0.0",
        "path": file,
        "schema": [
            "https://schemas.stacspec.org/v1.0.0/item-spec/json-schema/item.json"
        ],
        "valid_stac": True,
        "asset_type": "ITEM",
        "validation_method": "default",
        "assets_validated": {
            "format_valid": [
                "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2_analytic.tif",
                "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.jpg",
                "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.tif",
                "https://storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2_analytic_udm.tif",
                "http://remotedata.io/catalog/20201211_223832_CS2/extended-metadata.json",
                "http://cool-sat.com/catalog/20201211_223832_CS2/20201211_223832_CS2.EPH",
            ],
            "format_invalid": [],
            "request_valid": [],
            "request_invalid": [],
        },
    }


def test_linter_bad_assets():
    file = "sample_files/1.0.0/core-item-bad-links.json"
    linter = Linter(file, assets=True)
    asset_format_errors = [
        "https:/storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.jpg"
    ]
    asset_request_errors = [
        "https:/storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.jpg",
        "http://remotedata.io/catalog/20201211_223832_CS2/extended-metadata.json",
        "http://cool-sat.com/catalog/20201211_223832_CS2/20201211_223832_CS2.EPH",
    ]
    assert linter.version == "1.0.0"
    assert linter.valid_stac == True
    assert linter.asset_type == "ITEM"
    assert linter.invalid_asset_format == asset_format_errors
    assert linter.invalid_asset_request == asset_request_errors


def test_linter_bad_links():
    file = "sample_files/1.0.0/core-item-bad-links.json"
    linter = Linter(file, links=True)
    link_format_errors = ["http:/remotdata.io/catalog/20201211_223832_CS2/index.html"]
    link_request_errors = [
        "http://catalog/collection.json",
        "http:/remotdata.io/catalog/20201211_223832_CS2/index.html",
    ]
    assert linter.version == "1.0.0"
    assert linter.valid_stac == True
    assert linter.asset_type == "ITEM"
    assert len(linter.invalid_link_format) > 0
    assert linter.invalid_link_format == link_format_errors
    assert linter.invalid_link_request == link_request_errors


def test_linter_bad_links_assets():
    file = "sample_files/1.0.0/core-item-bad-links.json"
    linter = Linter(file, assets=True, links=True)
    asset_format_errors = [
        "https:/storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.jpg"
    ]
    asset_request_errors = [
        "https:/storage.googleapis.com/open-cogs/stac-examples/20201211_223832_CS2.jpg",
        "http://remotedata.io/catalog/20201211_223832_CS2/extended-metadata.json",
        "http://cool-sat.com/catalog/20201211_223832_CS2/20201211_223832_CS2.EPH",
    ]
    link_format_errors = [
        "http:/remotdata.io/catalog/20201211_223832_CS2/index.html",
    ]
    link_request_errors = [
        "http://catalog/collection.json",
        "http:/remotdata.io/catalog/20201211_223832_CS2/index.html",
    ]
    assert linter.version == "1.0.0"
    assert linter.valid_stac == True
    assert linter.asset_type == "ITEM"
    assert len(linter.invalid_link_format) > 0
    assert linter.invalid_asset_format == asset_format_errors
    assert linter.invalid_asset_request == asset_request_errors
    assert linter.invalid_link_format == link_format_errors
    assert linter.invalid_link_request == link_request_errors



from stac_check.lint import Linter


def test_linter_collection_recursive():
    file = "sample_files/1.0.0/catalog-with-bad-item.json"
    linter = Linter(file, assets=False, links=False, recursive=True)
    assert linter.version == "1.0.0"
    assert linter.recursive == True
    msg = linter.validate_all[0]
    assert msg["valid_stac"] is False
    assert msg["error_type"] == "JSONSchemaValidationError"
    # Accept either 'message' or 'error_message' as the error string
    error_msg = msg.get("error_message") or msg.get("message", "")
    assert "'id' is a required property" in error_msg
    # Optionally check path, version, schema if present
    if "path" in msg:
        assert msg["path"].endswith("bad-item.json")


def test_linter_recursive_max_depth_1():
    file = "https://radarstac.s3.amazonaws.com/stac/catalog.json"
    stac = Linter(file, assets=False, links=False, recursive=True, max_depth=1)
    assert stac.validate_all == [
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/catalog.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/catalog.json"],
            "asset_type": "CATALOG",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        }
    ]


def test_linter_recursive_max_depth_4():
    file = "https://radarstac.s3.amazonaws.com/stac/catalog.json"
    stac = Linter(file, assets=False, links=False, recursive=True, max_depth=4)
    assert stac.validate_all == [
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/catalog.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/catalog.json"],
            "asset_type": "CATALOG",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/collection.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/collection.json"],
            "asset_type": "COLLECTION",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/slc/catalog.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/catalog.json"],
            "asset_type": "CATALOG",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/slc/2012-05-13/RS1_M0630938_F2N_20120513_225708_HH_SLC.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/item.json"],
            "asset_type": "ITEM",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/slc/2012-06-14/RS1_M0634796_F3F_20120614_110317_HH_SLC.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/item.json"],
            "asset_type": "ITEM",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/slc/2012-06-14/RS1_M0634795_F3F_20120614_110311_HH_SLC.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/item.json"],
            "asset_type": "ITEM",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/slc/2012-10-12/RS1_M0634798_F3F_20121012_110325_HH_SLC.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/item.json"],
            "asset_type": "ITEM",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/slc/2012-10-12/RS1_M0634799_F3F_20121012_110331_HH_SLC.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/item.json"],
            "asset_type": "ITEM",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/raw/catalog.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/catalog.json"],
            "asset_type": "CATALOG",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
        {
            "version": "0.7.0",
            "path": "https://radarstac.s3.amazonaws.com/stac/radarsat-1/raw/2012-05-13/RS1_M0000676_F2N_20120513_225701_HH_RAW.json",
            "schema": ["https://cdn.staclint.com/v0.7.0/item.json"],
            "asset_type": "ITEM",
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "valid_stac": True,
        },
    ]


def test_linter_recursive_100():
    file = "https://digital-atlas.s3.amazonaws.com/stac/public_stac/population/worldpop_2020/collection.json"
    stac = Linter(file, assets=False, links=False, recursive=True, max_depth=4)
    assert stac.validate_all == [
        {
            "asset_type": "COLLECTION",
            "path": "https://digital-atlas.s3.amazonaws.com/stac/public_stac/population/worldpop_2020/collection.json",
            "schema": [
                "https://stac-extensions.github.io/scientific/v1.0.0/schema.json",
                "https://schemas.stacspec.org/v1.0.0/collection-spec/json-schema/collection.json",
            ],
            "valid_stac": True,
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "version": "1.0.0",
        },
        {
            "asset_type": "ITEM",
            "path": "https://digital-atlas.s3.amazonaws.com/stac/public_stac/population/worldpop_2020/./pop_2020/pop_2020.json",
            "schema": [
                "https://stac-extensions.github.io/projection/v1.1.0/schema.json",
                "https://stac-extensions.github.io/file/v2.1.0/schema.json",
                "https://stac-extensions.github.io/raster/v1.1.0/schema.json",
                "https://schemas.stacspec.org/v1.0.0/item-spec/json-schema/item.json",
            ],
            "valid_stac": True,
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "version": "1.0.0",
        },
        {
            "asset_type": "ITEM",
            "path": "https://digital-atlas.s3.amazonaws.com/stac/public_stac/population/worldpop_2020/./popDens_2020/popDens_2020.json",
            "schema": [
                "https://stac-extensions.github.io/projection/v1.1.0/schema.json",
                "https://stac-extensions.github.io/file/v2.1.0/schema.json",
                "https://stac-extensions.github.io/raster/v1.1.0/schema.json",
                "https://schemas.stacspec.org/v1.0.0/item-spec/json-schema/item.json",
            ],
            "valid_stac": True,
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "version": "1.0.0",
        },
        {
            "asset_type": "ITEM",
            "path": "https://digital-atlas.s3.amazonaws.com/stac/public_stac/population/worldpop_2020/./pop_2020_parquet/pop_2020_parquet.json",
            "schema": [
                "https://stac-extensions.github.io/table/v1.2.0/schema.json",
                "https://stac-extensions.github.io/file/v2.1.0/schema.json",
                "https://schemas.stacspec.org/v1.0.0/item-spec/json-schema/item.json",
            ],
            "valid_stac": True,
            "validation_method": "recursive",
            "validator_engine": "jsonschema",
            "version": "1.0.0",
        },
    ]



import importlib.metadata
import sys
from typing import Optional

import click

from stac_check.api_lint import ApiLinter
from stac_check.display_messages import (
    cli_message,
    collections_message,
    intro_message,
    item_collection_message,
    recursive_message,
)
from stac_check.lint import Linter
from stac_check.utilities import handle_output


@click.option(
    "--collections",
    is_flag=True,
    help="Validate collections endpoint response. Can be combined with --pages. Defaults to one page.",
)
@click.option(
    "--item-collection",
    is_flag=True,
    help="Validate item collection response. Can be combined with --pages. Defaults to one page.",
)
@click.option(
    "--pages",
    "-p",
    type=int,
    help="Maximum number of pages to validate via --item-collection or --collections. Defaults to one page.",
)
@click.option(
    "--recursive",
    "-r",
    is_flag=True,
    help="Recursively validate all related stac objects.",
)
@click.option(
    "--max-depth",
    "-m",
    type=int,
    help="Maximum depth to traverse when recursing. Omit this argument to get full recursion. Ignored if `recursive == False`.",
)
@click.option(
    "-a", "--assets", is_flag=True, help="Validate assets for format and response."
)
@click.option(
    "-l", "--links", is_flag=True, help="Validate links for format and response."
)
@click.option(
    "--output",
    "-o",
    type=click.Path(dir_okay=False, writable=True),
    help="Save output to the specified file. Only works with --collections, --item-collection, or --recursive.",
)
@click.option(
    "--no-assets-urls",
    is_flag=True,
    help="Disables the opening of href links when validating assets (enabled by default).",
)
@click.option(
    "--header",
    type=(str, str),
    multiple=True,
    help="HTTP header to include in the requests. Can be used multiple times.",
)
@click.option(
    "--pydantic",
    is_flag=True,
    help="Use pydantic validation (requires stac-pydantic to be installed).",
)
@click.option(
    "--verbose",
    "-v",
    is_flag=True,
    help="Enable verbose output.",
)
@click.command()
@click.argument("file")
@click.version_option(version=importlib.metadata.distribution("stac-check").version)
def main(
    file: str,
    collections: bool,
    item_collection: bool,
    pages: Optional[int],
    recursive: bool,
    max_depth: Optional[int],
    assets: bool,
    links: bool,
    no_assets_urls: bool,
    header: tuple[tuple[str, str], ...],
    pydantic: bool,
    verbose: bool,
    output: Optional[str],
) -> None:
    """Main entry point for the stac-check CLI.

    Args:
        file: The STAC file or URL to validate
        collections: Validate a collections endpoint
        item_collection: Validate an item collection
        pages: Number of pages to validate (for API endpoints)
        recursive: Recursively validate linked STAC objects
        max_depth: Maximum depth for recursive validation
        assets: Validate assets
        links: Validate links
        no_assets_urls: Disable URL validation for assets
        header: Additional HTTP headers
        pydantic: Use stac-pydantic for validation
        verbose: Show verbose output
        output: Save output to file (only with --collections, --item-collection, or --recursive)
    """
    # Check if output is used without --collections, --item-collection, or --recursive
    if output and not any([collections, item_collection, recursive]):
        click.echo(
            "Error: --output can only be used with --collections, --item-collection, or --recursive",
            err=True,
        )
        sys.exit(1)
    # Check if pydantic validation is requested but not installed
    if pydantic:
        try:
            importlib.import_module("stac_pydantic")
        except ImportError:
            click.secho(
                "Warning: stac-pydantic is not installed. Pydantic validation will be disabled.\n"
                "To enable pydantic validation, install it with: pip install stac-check[pydantic]",
                fg="yellow",
            )
            pydantic = False

    if collections or item_collection:
        # Handle API-based validation (collections or item collections)
        api_linter = ApiLinter(
            source=file,
            object_list_key="collections" if collections else "features",
            pages=pages if pages else 1,
            headers=dict(header),
            verbose=verbose,
        )
        results = api_linter.lint_all()

        # Create a dummy Linter instance for display purposes
        display_linter = Linter(
            file,
            assets=assets,
            links=links,
            headers=dict(header),
            pydantic=pydantic,
            verbose=verbose,
        )

        # Show intro message in the terminal
        intro_message(display_linter)

        # Define output generation function (without intro message since we already showed it)
        def generate_output():
            if collections:
                collections_message(
                    api_linter,
                    results=results,
                    cli_message_func=cli_message,
                    verbose=verbose,
                )
            elif item_collection:
                item_collection_message(
                    api_linter,
                    results=results,
                    cli_message_func=cli_message,
                    verbose=verbose,
                )

        # Handle output (without duplicating the intro message)
        handle_output(output, generate_output)
        sys.exit(0 if all(msg.get("valid_stac") is True for msg in results) else 1)
    else:
        # Handle file-based validation (single file or recursive)
        linter = Linter(
            file,
            assets=assets,
            links=links,
            recursive=recursive,
            max_depth=max_depth,
            assets_open_urls=not no_assets_urls,
            headers=dict(header),
            pydantic=pydantic,
            verbose=verbose,
        )

        intro_message(linter)

        # Define output generation function (without intro message since we already showed it)
        def generate_output():
            if recursive:
                recursive_message(linter, cli_message_func=cli_message, verbose=verbose)
            else:
                cli_message(linter)

        # Handle output (without duplicating the intro message)
        handle_output(output if recursive else None, generate_output)

        sys.exit(0 if linter.valid_stac else 1)



from unittest import mock

import pytest

from stac_check.api_lint import ApiLinter
from stac_check.display_messages import (
    cli_message,
    collections_message,
    item_collection_message,
)


@pytest.fixture
def item_collection_url():
    """URL for a real STAC API item collection endpoint."""
    return "https://stac.geobon.org/collections/chelsa-clim/items"


@pytest.fixture
def collections_url():
    """URL for a real STAC API collections endpoint."""
    return "https://stac.geobon.org/collections"


@pytest.fixture
def invalid_url():
    """URL for a non-existent endpoint to test error handling."""
    return "https://example.com/not-a-real-endpoint"


def test_item_collection_validation(item_collection_url):
    """Test item collection validation with a real STAC API endpoint."""
    linter = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=1,
    )

    results = linter.lint_all()

    # Verify we got results
    assert len(results) > 0
    # Check that items are valid STAC
    assert all(result["valid_stac"] for result in results)
    # Check that we have original objects
    assert all("original_object" in result for result in results)

    # Check specific fields in the results
    for result in results:
        assert "path" in result
        assert "version" in result
        assert "schema" in result
        assert "asset_type" in result
        assert result["asset_type"] == "ITEM"


def test_item_collection_with_pages(item_collection_url):
    """Test item collection validation with pages parameter."""
    # Test with 1 page
    linter_1page = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=1,
    )
    results_1page = linter_1page.lint_all()

    # Test with 2 pages
    linter_2pages = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=2,
    )
    results_2pages = linter_2pages.lint_all()

    # Verify we got more results with 2 pages than with 1 page
    # This assumes the API returns paginated results
    assert len(results_2pages) >= len(results_1page)

    # Check that the pages parameter was respected
    assert linter_1page.pages == 1
    assert linter_2pages.pages == 2


def test_collections_validation(collections_url):
    """Test collections validation with a real STAC API endpoint."""
    linter = ApiLinter(
        source=collections_url,
        object_list_key="collections",
        pages=1,
    )

    results = linter.lint_all()

    # Verify we got results
    assert len(results) > 0
    # Check that we have original objects
    assert all("original_object" in result for result in results)
    # Check that all collections have an ID
    assert all("id" in result["original_object"] for result in results)

    # Check specific fields in the results
    for result in results:
        assert "path" in result
        assert "version" in result
        assert "schema" in result


def test_api_linter_initialization(item_collection_url, collections_url):
    """Test that the ApiLinter can be initialized with different parameters."""
    # Test with item collection
    item_linter = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=1,
    )
    assert item_linter.object_list_key == "features"
    assert item_linter.pages == 1
    assert item_linter.source == item_collection_url

    # Test with collections
    collections_linter = ApiLinter(
        source=collections_url,
        object_list_key="collections",
        pages=2,
    )
    assert collections_linter.object_list_key == "collections"
    assert collections_linter.pages == 2
    assert collections_linter.source == collections_url

    # Test with custom headers
    headers = {"X-Custom-Header": "test-value"}
    linter_with_headers = ApiLinter(
        source=collections_url,
        object_list_key="collections",
        pages=1,
        headers=headers,
    )
    assert linter_with_headers.headers == headers


def test_error_handling_with_individual_items():
    """Test that ApiLinter handles errors with individual items gracefully."""
    # Create a mock that returns a valid response but with an item that will cause an error
    with mock.patch("stac_check.api_lint.fetch_and_parse_file") as mock_fetch:
        # Return a response with a malformed item that will cause an exception during validation
        mock_fetch.return_value = {
            "features": [
                {
                    "id": "valid-item",
                    "type": "Feature",
                    "geometry": {},
                    "properties": {},
                },
                # This item is missing required fields and will cause validation to fail
                {"id": "invalid-item"},
            ]
        }

        linter = ApiLinter(
            source="https://example.com/items",
            object_list_key="features",
            pages=1,
        )

        # This should not raise an exception
        results = linter.lint_all()

        # Should return results for both items
        assert len(results) == 2

        # Find the invalid item result
        invalid_result = next(r for r in results if r["path"].endswith("invalid-item"))

        # Check that it was marked as invalid
        assert invalid_result["valid_stac"] is False
        # Should have an error type
        assert "error_type" in invalid_result
        # Should have an error message
        assert "error_message" in invalid_result


def test_item_collection_message_display(item_collection_url):
    """Test that item_collection_message correctly displays validation results."""
    linter = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=1,
    )

    results = linter.lint_all()

    # Mock click.secho to verify it's called with the right arguments
    with mock.patch("stac_check.display_messages.click.secho") as mock_secho:
        item_collection_message(linter, results, cli_message_func=cli_message)

        # Verify that click.secho was called with expected title
        # The actual title in the code is "Item Collection: Validate all assets in a feature collection"
        mock_secho.assert_any_call(
            "Item Collection: Validate all assets in a feature collection", bold=True
        )
        # Verify that pages info was displayed - the format is "Pages = 1" not "Pages: 1"
        mock_secho.assert_any_call("Pages = 1")


def test_collections_message_display(collections_url):
    """Test that collections_message correctly displays validation results."""
    linter = ApiLinter(
        source=collections_url,
        object_list_key="collections",
        pages=1,
    )

    results = linter.lint_all()

    # Mock click.secho to verify it's called with the right arguments
    with mock.patch("stac_check.display_messages.click.secho") as mock_secho:
        collections_message(linter, results, cli_message_func=cli_message)

        # Verify that click.secho was called with expected title
        mock_secho.assert_any_call(
            "Collections: Validate all collections in a STAC API", bold=True
        )
        # Verify that pages info was displayed - the format is "Pages = 1" not "Pages: 1"
        mock_secho.assert_any_call("Pages = 1")


def test_pages_parameter_handling(item_collection_url):
    """Test that the pages parameter correctly limits pagination."""
    # Test with explicit pages=1
    linter_explicit_1 = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=1,
    )
    results_explicit_1 = linter_explicit_1.lint_all()

    # Test with None (should default to 1)
    linter_none = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=None,
    )
    results_none = linter_none.lint_all()

    # Test with default (should be 1)
    linter_default = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
    )
    results_default = linter_default.lint_all()

    # Test with pages=2
    linter_2pages = ApiLinter(
        source=item_collection_url,
        object_list_key="features",
        pages=2,
    )
    results_2pages = linter_2pages.lint_all()

    # Verify internal pages parameter is set correctly
    assert linter_explicit_1.pages == 1
    assert linter_none.pages == 1  # Should default to 1
    assert linter_default.pages == 1  # Should default to 1
    assert linter_2pages.pages == 2

    # Verify results count is consistent for all 1-page variants
    assert len(results_explicit_1) == len(results_none)
    assert len(results_explicit_1) == len(results_default)

    # Verify we get more results with 2 pages (if API supports pagination)
    # This test might need to be adjusted if the API doesn't have enough items
    if len(results_2pages) > len(results_explicit_1):
        assert len(results_2pages) > len(results_explicit_1)



"""Tests for the stac-check CLI."""

import os
import tempfile
from unittest.mock import MagicMock, patch

import pytest
from click.testing import CliRunner

from stac_check.cli import main as cli_main


@pytest.fixture
def runner():
    """Fixture for invoking command-line interfaces."""
    return CliRunner()


def test_cli_help(runner):
    """Test the CLI help output."""
    result = runner.invoke(cli_main, ["--help"])
    assert result.exit_code == 0
    assert "Show this message and exit." in result.output


def test_cli_version(runner):
    """Test the --version flag."""
    result = runner.invoke(cli_main, ["--version"])
    assert result.exit_code == 0
    # The version output is in the format: main, version X.Y.Z
    assert "version" in result.output


def test_cli_validate_local_file(runner):
    """Test validating a local file."""
    test_file = os.path.join(
        os.path.dirname(__file__), "../sample_files/1.0.0/core-item.json"
    )
    result = runner.invoke(cli_main, [test_file])
    assert result.exit_code == 0
    assert "Passed: True" in result.output


def test_cli_validate_recursive(runner):
    """Test recursive validation."""
    test_dir = os.path.join(
        os.path.dirname(__file__), "../sample_files/1.0.0/catalog-with-bad-item.json"
    )
    result = runner.invoke(cli_main, [test_dir, "--recursive"])
    assert result.exit_code == 0
    assert "Assets Checked" in result.output


def test_cli_output_to_file(runner):
    """Test output to file with --output flag."""
    test_file = os.path.join(
        os.path.dirname(__file__), "../sample_files/1.0.0/core-item.json"
    )
    with tempfile.NamedTemporaryFile(delete=False) as tmp:
        output_file = tmp.name

    try:
        result = runner.invoke(
            cli_main, [test_file, "--recursive", "--output", output_file]
        )
        assert result.exit_code == 0
        assert os.path.exists(output_file)
        with open(output_file, "r") as f:
            content = f.read()
            assert "Passed: True" in content
    finally:
        if os.path.exists(output_file):
            os.unlink(output_file)


def test_cli_collections(runner):
    """Test --collections flag with mock."""
    with patch("stac_check.cli.ApiLinter") as mock_api_linter, patch(
        "stac_check.cli.Linter"
    ) as mock_linter:
        # Mock ApiLinter instance
        mock_api_instance = MagicMock()
        mock_api_instance.lint_all.return_value = [{"valid_stac": True}]
        mock_api_linter.return_value = mock_api_instance

        # Mock Linter instance used for display
        mock_linter_instance = MagicMock()
        mock_linter.return_value = mock_linter_instance

        result = runner.invoke(
            cli_main,
            ["https://example.com/collections", "--collections", "--pages", "1"],
        )

        assert result.exit_code == 0
        mock_api_linter.assert_called_once_with(
            source="https://example.com/collections",
            object_list_key="collections",
            pages=1,
            headers={},
            verbose=False,
        )


def test_cli_item_collection(runner):
    """Test --item-collection flag with mock."""
    with patch("stac_check.cli.ApiLinter") as mock_api_linter, patch(
        "stac_check.cli.Linter"
    ) as mock_linter:
        # Mock ApiLinter instance
        mock_api_instance = MagicMock()
        mock_api_instance.lint_all.return_value = [{"valid_stac": True}]
        mock_api_linter.return_value = mock_api_instance

        # Mock Linter instance used for display
        mock_linter_instance = MagicMock()
        mock_linter.return_value = mock_linter_instance

        result = runner.invoke(
            cli_main, ["https://example.com/items", "--item-collection", "--pages", "2"]
        )

        assert result.exit_code == 0
        mock_api_linter.assert_called_once_with(
            source="https://example.com/items",
            object_list_key="features",
            pages=2,
            headers={},
            verbose=False,
        )


def test_cli_output_without_required_flags(runner):
    """Test that --output requires --collections, --item-collection, or --recursive."""
    with tempfile.NamedTemporaryFile() as tmp:
        result = runner.invoke(
            cli_main, ["https://example.com/catalog.json", "--output", tmp.name]
        )
        assert result.exit_code == 1
        assert (
            "--output can only be used with --collections, --item-collection, or --recursive"
            in result.output
        )


def test_cli_verbose_flag(runner):
    """Test that --verbose flag is passed correctly."""
    with patch("stac_check.cli.Linter") as mock_linter:
        mock_instance = MagicMock()
        mock_linter.return_value = mock_instance

        test_file = os.path.join(
            os.path.dirname(__file__), "../sample_files/1.0.0/core-item.json"
        )
        result = runner.invoke(cli_main, [test_file, "--verbose"])

        assert result.exit_code == 0
        mock_linter.assert_called_once()
        assert mock_linter.call_args[1]["verbose"] is True


def test_cli_headers(runner):
    """Test that custom headers are passed correctly."""
    with patch("stac_check.cli.Linter") as mock_linter:
        mock_instance = MagicMock()
        mock_linter.return_value = mock_instance

        test_file = os.path.join(
            os.path.dirname(__file__), "../sample_files/1.0.0/core-item.json"
        )
        # The header format is: --header KEY VALUE (space-separated, not colon-separated)
        result = runner.invoke(
            cli_main,
            [
                test_file,
                "--header",
                "Authorization",
                "Bearer token",
                "--header",
                "X-Custom",
                "value",
            ],
        )

        assert result.exit_code == 0
        mock_linter.assert_called_once()
        # The headers should be passed as a dictionary to the Linter
        headers = mock_linter.call_args[1]["headers"]
        assert isinstance(headers, dict)
        assert headers.get("Authorization") == "Bearer token"
        assert headers.get("X-Custom") == "value"


def test_cli_pydantic_flag(runner):
    """Test that the --pydantic flag is passed correctly."""
    with patch("stac_check.cli.Linter") as mock_linter, patch(
        "stac_check.cli.importlib.import_module"
    ):
        mock_instance = MagicMock()
        mock_linter.return_value = mock_instance

        test_file = os.path.join(
            os.path.dirname(__file__), "../sample_files/1.0.0/core-item.json"
        )

        # Test with --pydantic flag
        result = runner.invoke(cli_main, [test_file, "--pydantic"])

        assert result.exit_code == 0
        mock_linter.assert_called_once()
        # Check that pydantic=True was passed to Linter
        assert mock_linter.call_args[1]["pydantic"] is True

        # Test without --pydantic flag (should default to False)
        mock_linter.reset_mock()
        result = runner.invoke(cli_main, [test_file])

        assert result.exit_code == 0
        mock_linter.assert_called_once()
        assert mock_linter.call_args[1]["pydantic"] is False



from stac_check.lint import Linter


def test_geometry_coordinates_order():
    """Test the check_geometry_coordinates_order method for detecting potentially incorrectly ordered coordinates."""
    # Create a test item with coordinates in the correct order (longitude, latitude)
    correct_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-coordinates-correct",
        "bbox": [10.0, -10.0, 20.0, 10.0],
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [10.0, -10.0],  # lon, lat
                    [20.0, -10.0],
                    [20.0, 10.0],
                    [10.0, 10.0],
                    [10.0, -10.0],
                ]
            ],
        },
        "properties": {"datetime": "2023-01-01T00:00:00Z"},
    }

    # Create a test item with coordinates in the wrong order (latitude, longitude)
    # but with values that don't trigger the validation checks
    undetectable_reversed_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-coordinates-undetectable-reversed",
        "bbox": [10.0, -10.0, 20.0, 10.0],
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [-10.0, 10.0],  # lat, lon (reversed) but within valid ranges
                    [-10.0, 20.0],
                    [10.0, 20.0],
                    [10.0, 10.0],
                    [-10.0, 10.0],
                ]
            ],
        },
        "properties": {"datetime": "2023-01-01T00:00:00Z"},
    }

    # Create a test item with coordinates that are clearly reversed (latitude > 90)
    clearly_incorrect_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-coordinates-clearly-incorrect",
        "bbox": [10.0, -10.0, 20.0, 10.0],
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [10.0, 100.0],  # Second value (latitude) > 90
                    [20.0, 100.0],
                    [20.0, 100.0],
                    [10.0, 100.0],
                    [10.0, 100.0],
                ]
            ],
        },
        "properties": {"datetime": "2023-01-01T00:00:00Z"},
    }

    # Create a test item with coordinates that may be reversed based on heuristic
    # (first value > 90, second value < 90, first value > second value*2)
    heuristic_incorrect_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-coordinates-heuristic-incorrect",
        "bbox": [10.0, -10.0, 20.0, 10.0],
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [120.0, 40.0],  # First value > 90, second < 90, first > second*2
                    [120.0, 40.0],
                    [120.0, 40.0],
                    [120.0, 40.0],
                    [120.0, 40.0],
                ]
            ],
        },
        "properties": {"datetime": "2023-01-01T00:00:00Z"},
    }

    # Test with correct coordinates - this should pass both checks
    linter = Linter(correct_item)
    assert linter.check_geometry_coordinates_order() == True
    assert linter.check_geometry_coordinates_definite_errors() == True

    # Test with reversed coordinates that are within valid ranges
    # Current implementation can't detect this case, so both checks pass
    linter = Linter(undetectable_reversed_item)
    assert (
        linter.check_geometry_coordinates_order() == True
    )  # Passes because values are within valid ranges
    assert (
        linter.check_geometry_coordinates_definite_errors() == True
    )  # Passes because values are within valid ranges

    # Test with clearly incorrect coordinates (latitude > 90)
    # This should fail the definite errors check but pass the order check (which now only uses heuristic)
    linter = Linter(clearly_incorrect_item)
    assert (
        linter.check_geometry_coordinates_order() == True
    )  # Now passes because it only checks heuristic

    # Check that definite errors are detected
    result = linter.check_geometry_coordinates_definite_errors()
    assert result is not True  # Should not be True
    assert isinstance(result, tuple)  # Should be a tuple
    assert result[0] is False  # First element should be False
    assert len(result[1]) > 0  # Should have at least one invalid coordinate
    assert result[1][0][1] == 100.0  # The latitude value should be 100.0
    assert "latitude > ±90°" in result[1][0][2]  # Should indicate latitude error

    # Test with coordinates that trigger the heuristic
    # This should fail the order check but pass the definite errors check
    linter = Linter(heuristic_incorrect_item)
    assert (
        linter.check_geometry_coordinates_order() == False
    )  # Fails because of heuristic
    assert (
        linter.check_geometry_coordinates_definite_errors() == True
    )  # Passes because values are within valid ranges

    # Test that the best practices dictionary contains the appropriate error messages
    best_practices = linter.create_best_practices_dict()

    # For heuristic-based detection
    linter = Linter(heuristic_incorrect_item)
    best_practices = linter.create_best_practices_dict()
    assert "geometry_coordinates_order" in best_practices
    assert (
        "may be in the wrong order" in best_practices["geometry_coordinates_order"][0]
    )

    # For definite errors detection
    linter = Linter(clearly_incorrect_item)
    best_practices = linter.create_best_practices_dict()
    assert "geometry_coordinates_definite_errors" in best_practices
    assert (
        "contain invalid values"
        in best_practices["geometry_coordinates_definite_errors"][0]
    )


def test_bbox_antimeridian():
    """Test the check_bbox_antimeridian method for detecting incorrectly formatted bboxes that cross the antimeridian."""
    # Create a test item with an incorrectly formatted bbox that belts the globe
    # instead of properly crossing the antimeridian
    incorrect_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-antimeridian-incorrect",
        "bbox": [
            -170.0,  # west
            -10.0,  # south
            170.0,  # east (incorrect: this belts the globe instead of crossing the antimeridian)
            10.0,  # north
        ],
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [170.0, -10.0],
                    [-170.0, -10.0],
                    [-170.0, 10.0],
                    [170.0, 10.0],
                    [170.0, -10.0],
                ]
            ],
        },
        "properties": {"datetime": "2023-01-01T00:00:00Z"},
    }

    # Create a test item with a correctly formatted bbox that crosses the antimeridian
    # (west > east for antimeridian crossing)
    correct_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-antimeridian-correct",
        "bbox": [
            170.0,  # west
            -10.0,  # south
            -170.0,  # east (west > east indicates antimeridian crossing)
            10.0,  # north
        ],
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [170.0, -10.0],
                    [-170.0, -10.0],
                    [-170.0, 10.0],
                    [170.0, 10.0],
                    [170.0, -10.0],
                ]
            ],
        },
        "properties": {"datetime": "2023-01-01T00:00:00Z"},
    }

    # Test with the incorrect item (belting the globe)
    linter = Linter(incorrect_item)
    # The check should return False for the incorrectly formatted bbox
    assert linter.check_bbox_antimeridian() == False

    # Verify that the best practices dictionary contains the appropriate message
    best_practices = linter.create_best_practices_dict()
    assert "check_bbox_antimeridian" in best_practices
    assert len(best_practices["check_bbox_antimeridian"]) == 2

    # Check that the error messages include the west and east longitude values
    west_val = incorrect_item["bbox"][0]
    east_val = incorrect_item["bbox"][2]
    assert (
        f"(found west={west_val}, east={east_val})"
        in best_practices["check_bbox_antimeridian"][0]
    )

    # Test with the correct item - this should pass
    linter = Linter(correct_item)
    # The check should return True for the correctly formatted bbox
    assert linter.check_bbox_antimeridian() == True

    # Test with a normal bbox that doesn't cross the antimeridian
    normal_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-normal-bbox",
        "bbox": [10.0, -10.0, 20.0, 10.0],  # west  # south  # east  # north
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [10.0, -10.0],
                    [20.0, -10.0],
                    [20.0, 10.0],
                    [10.0, 10.0],
                    [10.0, -10.0],
                ]
            ],
        },
        "properties": {"datetime": "2023-01-01T00:00:00Z"},
    }

    # Test with a normal bbox - this should pass
    linter = Linter(normal_item)
    assert linter.check_bbox_antimeridian() == True


def test_bbox_matches_geometry():
    # Test with matching bbox and geometry
    file = "sample_files/1.0.0/core-item.json"
    linter = Linter(file)
    assert linter.check_bbox_matches_geometry() is True

    # Test with mismatched bbox and geometry
    mismatched_item = {
        "stac_version": "1.0.0",
        "stac_extensions": [],
        "type": "Feature",
        "id": "test-item",
        "bbox": [100.0, 0.0, 105.0, 1.0],  # Deliberately wrong bbox
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [172.91173669923782, 1.3438851951615003],
                    [172.95469614953714, 1.3438851951615003],
                    [172.95469614953714, 1.3690476620161975],
                    [172.91173669923782, 1.3690476620161975],
                    [172.91173669923782, 1.3438851951615003],
                ]
            ],
        },
        "properties": {"datetime": "2020-12-11T22:38:32.125Z"},
    }
    linter = Linter(mismatched_item)
    result = linter.check_bbox_matches_geometry()

    # Check that the result is a tuple and the first element is False
    assert isinstance(result, tuple)
    assert result[0] is False

    # Check that the tuple contains the expected elements (calculated bbox, actual bbox, differences)
    assert len(result) == 4
    calc_bbox, actual_bbox, differences = result[1], result[2], result[3]

    # Verify the calculated bbox matches the geometry coordinates
    assert calc_bbox == [
        172.91173669923782,
        1.3438851951615003,
        172.95469614953714,
        1.3690476620161975,
    ]

    # Verify the actual bbox is what we provided
    assert actual_bbox == [100.0, 0.0, 105.0, 1.0]

    # Verify the differences are calculated correctly
    expected_differences = [abs(actual_bbox[i] - calc_bbox[i]) for i in range(4)]
    assert differences == expected_differences

    # Test with null geometry (should return True as check is not applicable)
    null_geom_item = {
        "stac_version": "1.0.0",
        "type": "Feature",
        "id": "test-item-null-geom",
        "bbox": [100.0, 0.0, 105.0, 1.0],
        "geometry": None,
        "properties": {"datetime": "2020-12-11T22:38:32.125Z"},
    }
    linter = Linter(null_geom_item)
    assert linter.check_bbox_matches_geometry() is True

    # Test with missing bbox (should return True as check is not applicable)
    no_bbox_item = {
        "stac_version": "1.0.0",
        "type": "Feature",
        "id": "test-item-no-bbox",
        "geometry": {
            "type": "Polygon",
            "coordinates": [
                [
                    [172.91173669923782, 1.3438851951615003],
                    [172.95469614953714, 1.3438851951615003],
                    [172.95469614953714, 1.3690476620161975],
                    [172.91173669923782, 1.3690476620161975],
                    [172.91173669923782, 1.3438851951615003],
                ]
            ],
        },
        "properties": {"datetime": "2020-12-11T22:38:32.125Z"},
    }
    linter = Linter(no_bbox_item)
    assert linter.check_bbox_matches_geometry() is True



import json

import requests_mock

from stac_check.lint import Linter


def test_linter_collection():
    file = "sample_files/1.0.0/collection.json"
    linter = Linter(file, assets=False, links=False)
    assert linter.version == "1.0.0"
    assert linter.valid_stac == True
    assert linter.asset_type == "COLLECTION"
    assert linter.check_summaries() == True


def test_linter_collection_no_summaries():
    file = "sample_files/1.0.0/collection-no-summaries.json"
    linter = Linter(file, assets=False, links=False)
    assert linter.version == "1.0.0"
    assert linter.valid_stac == True
    assert linter.asset_type == "COLLECTION"
    assert linter.check_summaries() == False
    assert linter.best_practices_msg == [
        "STAC Best Practices: ",
        "Object should be called 'collection.json' not 'collection-no-summaries.json'",
        "",
        "A STAC collection should contain a summaries field",
        "It is recommended to store information like eo:bands in summaries",
        "",
    ]


def test_linter_catalog():
    file = "sample_files/1.0.0/catalog.json"
    linter = Linter(file, assets=False, links=False)
    assert linter.version == "1.0.0"
    assert linter.valid_stac == True
    assert linter.asset_type == "CATALOG"
    assert linter.check_bloated_links() == False


def test_linter_item_id_not_matching_file_name():
    file = "sample_files/1.0.0/core-item.json"
    linter = Linter(file)
    assert linter.file_name == "core-item"
    assert linter.object_id == "20201211_223832_CS2"
    assert linter.file_name != linter.object_id
    assert linter.check_item_id_file_name() == False


def test_linter_collection_catalog_id():
    file = "sample_files/1.0.0/collection-no-title.json"
    linter = Linter(file)
    assert linter.check_catalog_file_name() == False


def test_linter_item_id_format_best_practices():
    file = "sample_files/1.0.0/core-item-invalid-id.json"
    linter = Linter(file)
    assert linter.check_searchable_identifiers() == False
    assert linter.check_percent_encoded() == True


def test_datetime_set_to_null():
    file = "sample_files/1.0.0/core-item-null-datetime.json"
    linter = Linter(file)
    assert linter.check_datetime_null() == True


def test_unlocated_item():
    file = "sample_files/1.0.0/core-item-unlocated.json"
    linter = Linter(file)
    assert linter.check_unlocated() == True
    assert linter.check_geometry_null() == True

    file = "sample_files/1.0.0/core-item-unlocated-null-bbox.json"
    linter = Linter(file)
    assert linter.check_unlocated() == False
    assert linter.check_geometry_null() == True


def test_bloated_item():
    file = "sample_files/1.0.0/core-item-bloated.json"
    linter = Linter(file)

    assert linter.check_bloated_metadata() == True
    assert len(linter.data["properties"]) > 20

    assert linter.check_bloated_links() == True
    assert len(linter.data["links"]) > 20


def test_small_thumbnail():
    file = "sample_files/1.0.0/core-item-large-thumbnail.json"
    linter = Linter(file)

    assert linter.check_thumbnail() != True

    file = "sample_files/1.0.0/core-item.json"
    linter = Linter(file)

    assert linter.check_thumbnail() == True


def test_title_field():
    file = "sample_files/1.0.0/collection-no-title.json"
    linter = Linter(file)

    assert linter.check_links_title_field() == False


def test_self_in_links():
    file = "sample_files/1.0.0/collection-no-title.json"
    linter = Linter(file)
    assert linter.check_links_self() == False


def test_catalog_name():
    file = "sample_files/1.0.0/catalog.json"
    linter = Linter(file)
    assert linter.check_catalog_file_name()
    file = "sample_files/1.0.0/collection.json"
    linter = Linter(file)
    assert linter.check_catalog_file_name()


def test_lint_header():
    file = "sample_files/1.0.0/core-item.json"
    url = "https://localhost/" + file

    no_headers = {}
    valid_headers = {"x-api-key": "a-valid-api-key"}

    with requests_mock.Mocker(real_http=True) as mock, open(file) as json_data:
        mock.get(url, request_headers=no_headers, status_code=403, json={})
        mock.get(url, request_headers=valid_headers, json=json.load(json_data))

        linter = Linter(url, assets=False, headers=valid_headers)
        assert linter.message == {
            "version": "1.0.0",
            "path": "https://localhost/sample_files/1.0.0/core-item.json",
            "schema": [
                "https://schemas.stacspec.org/v1.0.0/item-spec/json-schema/item.json"
            ],
            "valid_stac": True,
            "asset_type": "ITEM",
            "validation_method": "default",
        }

        linter = Linter(url, assets=False, headers=no_headers)
        msg = linter.message
        assert msg["valid_stac"] is False
        assert msg["error_type"] == "HTTPError"
        # Accept either 'message' or 'error_message' as the error string
        error_msg = msg.get("error_message") or msg.get("message")
        assert (
            error_msg
            == "403 Client Error: None for url: https://localhost/sample_files/1.0.0/core-item.json"
        )
        # Optionally check path, version, schema if present
        if "path" in msg:
            assert msg["path"] == "https://localhost/sample_files/1.0.0/core-item.json"


def test_verbose_error_message():
    """Test that verbose error messages are properly formatted and included."""
    # Test with a known bad item that will generate validation errors
    file = "sample_files/1.0.0/bad-item.json"
    linter = Linter(file, verbose=True)

    # Verify the item is invalid
    assert linter.valid_stac is False

    # Check that we have the expected error message
    assert "id" in linter.error_msg.lower()
    assert "required" in linter.error_msg.lower()

    # Check that verbose error message contains expected structure
    assert isinstance(linter.verbose_error_msg, dict)
    assert "validator" in linter.verbose_error_msg
    assert "path_in_schema" in linter.verbose_error_msg

    # Check specific parts of the verbose error message
    assert linter.verbose_error_msg.get("validator") == "required"

    # Check path_in_schema - it might contain both strings and integers
    path_in_schema = linter.verbose_error_msg.get("path_in_schema", [])
    assert any(isinstance(p, (str, int)) for p in path_in_schema)

    # Check that the error message is included in the string representation
    verbose_str = str(linter.verbose_error_msg)
    assert "required" in verbose_str
    assert "id" in verbose_str
